#!/usr/bin/env python
#
# Copyright 2009 Marcelo Boveto Shima.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA    02110-1301, USA.
#
# Author: marceloshima@gmail.com (Marcelo Boveto Shima)

import os
import sys
import subprocess
import dbus
import signal
import getpass
import gobject

from tacix.server.nx import NXException
from tacix.server.nx.apps import AppProgram
from tacix import util
from tacix.constants import *

import dbus.mainloop.glib
dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)

import SocketServer
import socket
import math
import pwd


class SUAppProgram(AppProgram):
    def __init__(self, user, logger, cmd=None, log_path=None, callback=None):
        self.user = user
        AppProgram.__init__(self, logger, cmd=cmd, log_path=log_path, callback=callback)

    def _run(self):
        if self.user is None:
            return
        pwd_user = pwd.getpwnam(self.user)
        if pwd_user is None:
            return

        new_env = {}
        new_env['USER'] = pwd_user.pw_name
        new_env['USERNAME'] = pwd_user.pw_name
        new_env['LOGNAME'] = pwd_user.pw_name
        new_env['HOME'] = pwd_user.pw_dir
        for env_var in ['LANG', 'LANGUAGE', 'PATH']:
            if env_var in os.environ:
                new_env[env_var] = os.environ[env_var]

        self.logger.debug(" ".join(self.cmd))
        self.cmd = ["/bin/su", "-p", "-c", " ".join(self.cmd), "-l", pwd_user.pw_name]
        AppProgram._run(self)


class VNCXSessionProgram(SUAppProgram):
    def __init__(self, user, callback, logger, options):
        cmd = ["/etc/X11/Xsession"]
        self.options = options
        SUAppProgram.__init__(self, user, logger, cmd=cmd, log_path='/var/log/tacix/xsession.log', callback=callback)

    def _run(self):
        """Start the session application (startkde/gnome-session/etc)."""
        self.logger.info("NXSessionApp ready, launching app")
        os.environ['DISPLAY'] = ":%s" % self.options['display']

        SUAppProgram._run(self)

class RDPVNCProgram(AppProgram):
    def __init__(self, logger, options):
        '''
        '''
        cmd = ["Xvnc",
                "-geometry", options['geometry'],
                "-depth", options['depth'],
                "-rfbauth", options['rfbauth'],
                ":%s" % options['display']]
        AppProgram.__init__(self, logger, cmd=cmd, log_path="/var/log/tacix/xrdp%s.log" % options['display'])

    def _run(self):
        AppProgram._run(self)

    def terminate(self):
        AppProgram.terminate(self)


class MyTCPHandler(SocketServer.BaseRequestHandler):
    """
    The RequestHandler class for our server.

    It is instantiated once per connection to the server, and must
    override the handle() method to implement communication to the
    client.
    """
    def __init__(self, request, client_address, server):
        self.sock = None
        self.logger = util.get_logger('xrdp', '/var/log/tacix/xrdp.log')
        SocketServer.BaseRequestHandler.__init__(self, request, client_address, server)

    def handle(self):
        # self.request is the TCP socket connected to the client
        data = self.request.recv(1024).strip()

        version = uint32_to_int_data(data, 0)
        size = uint32_to_int_data(data, 4)
        type = uint16_to_int_data(data, 8)

        user_start_pos = 10
        user_length = uint16_to_int_data(data, user_start_pos)
        username = data[user_start_pos + 2 : user_start_pos + 2 + user_length]

        pwd_start_pos = user_start_pos + 2 + user_length
        password_length = uint16_to_int_data(data, pwd_start_pos)
        password = data[pwd_start_pos + 2 : pwd_start_pos + 2 + password_length]

        width_start = pwd_start_pos + 2 + password_length
        width = uint16_to_int_data(data, width_start)

        height_start = width_start + 2
        height = uint16_to_int_data(data, height_start)

        bpp_start = height_start + 2
        bpp = uint16_to_int_data(data, bpp_start)

        out_version = 0
        out_size = 14
        out_cmd = 3
        out_data = 1
        out_display = 10

        ret = ""
        ret = ret + int_to_uint32(out_version)
        ret = ret + int_to_uint32(out_size)
        ret = ret + int_to_uint16(out_cmd)
        ret = ret + int_to_uint16(out_data)
        ret = ret + int_to_uint16(out_display)

        self.request.send(ret)
        #self.request.close()
        options = {'geometry':'%sx%s' % (width, height),
                'depth': '%s' % bpp,
                'rfbauth':'/tmp/sesman_passwd',
                'display':'%s' % out_display}
        rdpvnc = RDPVNCProgram(self.logger, options)

        rdpvnc._run()
        xsessionprogram = VNCXSessionProgram(username, rdpvnc.terminate, self.logger, options)
        xsessionprogram._run()
        return

        '''
        if self.sock == None:
            self.sock = socket.socket(
                    socket.AF_INET, socket.SOCK_STREAM)
            self.sock.connect(('localhost', 3349))

        sent = self.sock.send(data)
        data_sesman = self.sock.recv(1024)
        print data_sesman

        for char in data_sesman:
            print "%r" % char
            print "%s" % ord(char)

        print len(data_sesman)
        self.request.send(data_sesman)
        '''

def int_to_uint16(value):
    return chr(int(math.floor(value / 256))) + chr(int(value % 256))

def int_to_uint16_str(string, value): 
    string = string + chr(int(math.floor(value / 256)))
    string = string + chr(int(value % 256))
    return str

def int_to_uint32(value):
    return chr(int(math.floor(value / (256 * 256 * 256)))) + chr(int(math.floor(value / (256 * 256)))) + chr(int(math.floor(value / 256))) + chr(int(value % 256))

def int_to_uint32_str(string, value):
    string = string + chr(int(math.floor(value / (256 * 256 * 256))))
    string = string + chr(int(math.floor(value / (256 * 256))))
    string = string + chr(int(math.floor(value / 256)))
    string = string + chr(int(value % 256))
    return str

def uint16_to_int(chars):
    return (ord(chars[0]) * 256) + ord(chars[1])

def uint16_to_int_data(data, start):
    return uint16_to_int(data[start : start + 2])

def uint32_to_int(chars):
    return (ord(chars[0]) * 256 * 256 * 265) + (ord(chars[1]) * 256 * 256) + (ord(chars[2]) * 256) + ord(chars[3])

def uint32_to_int_data(data, start):
    return uint32_to_int(data[start : start + 4])

if __name__ == "__main__":
    HOST, PORT = "localhost", 3350

    # Create the server, binding to localhost on port 9999
    server = SocketServer.TCPServer((HOST, PORT), MyTCPHandler)

    # Activate the server; this will keep running until you
    # interrupt the program with Ctrl-C
    server.serve_forever()

