#!/usr/bin/env python
#
# Copyright 2009 Marcelo Boveto Shima.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA    02110-1301, USA.
#
# Author: marceloshima@gmail.com (Marcelo Boveto Shima)

import os
import random
import pwd

try:
    from hashlib import md5
except ImportError:
    from md5 import md5

from tacix.server.nx.display import *

default_options = {
                  }

class XRDPOptions:
    def __init__( self ):
        '''
           Create the options based on the default options
        '''
        self.private_options = {}
        self.options = {}

        self.options.update(default_options)
        self.calculate_options()

    def gen_cookie(self):
        return md5(str(random.random())).hexdigest()

    def calculate_options(self):
        '''
           Calculate some options
        '''
        self.options['display'] = str(find_next_available_display(self.options['display']))
        self.options['cookie'] = self.gen_cookie()
        return

    def get_default_option( self, property ):
        if property in default_options:
            return default_options[property]
        return None

    def get_property( self, property ):
        if property in self.options:
            return self.options[property]
        return None

    def get_property_secure(self, property):
        if property in self.options:
            return self.options[property]

        if property in self.private_options:
            return self.private_options[property]
        return None

    def get_properties( self ):
        ret = {}
        ret.update(self.options)
        return ret

    def get_properties_secure( self ):
        ret = {}
        ret.update(self.options)
        ret.update(self.private_options)
        return ret

    def set_property( self, property, value ):
        handler_name = '_property_%s_handler' % property
        try:
            handler_method = getattr(self, handler_name)
            ret = handler_method(property, value)
            if not ret:
                return
        except AttributeError, e:
            pass

        if not self._is_allowed(property, value):
            return

        self.options[property] = value

    def _is_allowed( property, value ):
        return True

